
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "xstd/h/iostream.h"
#include "xstd/h/iomanip.h"

#include "base/OLog.h"
#include "base/ILog.h"
#include "base/ObjId.h"


void ObjId::reset() {
	theWorld.clear();
	theViserv = theType = theName = -1;

	theTarget = -1;

	theFlags = 0;
}

int ObjId::hash() const {
	return abs(theWorld.hash() * theType + theName);
}

bool ObjId::direct() const {
	return !embedded();
}

bool ObjId::embedded() const {
	Assert(theWorld);
	return theWorld.mutant();
}

// /*first*/ cachable miss
bool ObjId::fill() const {
	return basic() && /* !repeat() &&*/ cachable() && !hit();
}

ostream &ObjId::printFlags(ostream &os) const {
	if (repeat())
		os << "rpt,";
	if (hot())
		os << "hot,";
	if (foreign())
		os << "foreign,";

	if (basic())
		os << "basic,";
	if (ims())
		os << "ims,";
	if (reload())
		os << "reload,";
	if (rediredReq())
		os << "rediredReq,";
	if (repToRedir())
		os << "repToRedir,";

	if (cachable())
		os << "chb,";
	if (hit())
		os << "hit";

	return os;
}

void ObjId::store(OLog &ol) const {
	ol << theWorld << theViserv << theName << theType
		<< theTarget;
	ol.puti((int)theFlags);
}

void ObjId::load(ILog &il) {
	il >> theWorld >> theViserv >> theName >> theType
		>> theTarget;
	theFlags = (unsigned)il.geti();
}
