
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_STAT_INTERVAL_RECORD_H
#define PG_STAT_INTERVAL_RECORD_H

#include "base/LogObj.h"
#include "base/LevelStat.h"
#include "base/HRStat.h"
#include "base/IcpStat.h"

// holds basic statistics for a given interval
class StatIntvlRec: public LogObj {
	public:
		StatIntvlRec();

		void restart();

		virtual OLog &store(OLog &log) const;
		virtual ILog &load(ILog &);
		bool sane() const;

		double reqRate() const; // req/sec
		double repRate() const; // rep/sec
		AggrStat repTime() const; // hit + miss
		AggrStat repSize() const; // hit + miss

		int xactCnt() const { return theXactCnt + theXactErrCnt; }
		BigSize totFillSize() const;
		int totFillCount() const;
		double errRatio() const;
		double errPercent() const;
		TmSzStat reps() const; // all successful xactions

		void keepLevels(const StatIntvlRec &prevIntvl);
		void concat(const StatIntvlRec &r); // sequential intervals
		void merge(const StatIntvlRec &r);  // concurrent intervals

		ostream &print(ostream &os, const String &pfx) const;
		void linePrint(ostream &os) const;

	protected:
		void join(const StatIntvlRec &r);

	public: /* read only, except for kids */

		/* HTTP */
		LevelStat theWaitLvl; // resources waiting requests
		LevelStat theXactLvl; // active xactions
		LevelStat theOpenLvl; // open connections
		LevelStat theEstbLvl; // estanlished connections

		AggrStat theConnLifeTm; // to get mean life time
		AggrStat theConnUseCnt; // xactions per connection

		/* these are only for basic xactions */
		HRStat theIdealHR;    // offered HR
		HRStat theRealHR;     // measured HR
		HRStat theChbR;       // cachablity ratio
		TmSzStat theFill;     // cachable misses

		/* special xactions */
		TmSzStat theRediredReq; // transactions due to redir replies
		TmSzStat theRepToRedir; // transactions resulted in redir replies
		TmSzStat theIms;      // IMS transactions
		TmSzStat theReload;   // Reload transactions

		int theXactCnt;       // successful xactions 
		int theXactErrCnt;    // xaction errors
		int theXactRetrCnt;   // xaction retries

		/* ICP */
		IcpStat theIcpStat;   // icp-related stats

		Time theDuration;     // actual intvl length, computed for us
};

#endif
