
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "base/StatPhaseRec.h"
#include "base/OLog.h"
#include "base/ILog.h"


StatPhaseRec::StatPhaseRec() {
}

OLog &StatPhaseRec::store(OLog &log) const {
	StatIntvlRec::store(log);
	return log 
		<< theName 
		<< theGoal
		<< theContType
		<< theConnClose
		<< theSockRdSzH << theSockWrSzH
		<< theBasicXacts << theRediredReqXacts << theRepToRedirXacts 
		<< theImsXacts << theReloadXacts
		<< theIcpXacts
		<< theOidGen
		<< theErrors
		;
}

ILog &StatPhaseRec::load(ILog &log) {
	StatIntvlRec::load(log);
	return log
		>> theName
		>> theGoal
		>> theContType
		>> theConnClose
		>> theSockRdSzH >> theSockWrSzH
		>> theBasicXacts >> theRediredReqXacts >> theRepToRedirXacts 
		>> theImsXacts >> theReloadXacts
		>> theIcpXacts
		>> theOidGen
		>> theErrors
		;
}

void StatPhaseRec::concat(const StatPhaseRec &r) {
	if (theName && r.theName && theName != r.theName)
		theName += "|" + r.theName;
	StatIntvlRec::concat(r);
	theGoal.concat(r.theGoal);
	join(r);
}

void StatPhaseRec::merge(const StatPhaseRec &r) {
	if (theName && r.theName && theName != r.theName)
		theName += "&" + r.theName;
	StatIntvlRec::merge(r);
	theGoal.merge(r.theGoal);
	join(r);
}

void StatPhaseRec::join(const StatPhaseRec &r) {
	if (!theName)
		theName = r.theName;

	theContType.add(r.theContType);
	theConnClose.add(r.theConnClose);
	theSockRdSzH.add(r.theSockRdSzH);
	theSockWrSzH.add(r.theSockWrSzH);
	theBasicXacts += r.theBasicXacts;
	theRediredReqXacts += r.theRediredReqXacts;
	theRepToRedirXacts += r.theRepToRedirXacts;
	theImsXacts += r.theImsXacts;
	theReloadXacts += r.theReloadXacts;
	theIcpXacts  += r.theIcpXacts;
	theOidGen += r.theOidGen;
	theErrors.add(r.theErrors);
}

ostream &StatPhaseRec::print(ostream &os, const String &pfx) const {
	os << "name:\t " << theName << endl;

	StatIntvlRec::print(os, pfx);

	theGoal.print(os, pfx + "goal.");

	TmSzHistStat imsAll;
	imsAll += theImsXacts.hits();
	imsAll += theImsXacts.misses();

	TmSzHistStat basicAll;
	basicAll += theBasicXacts.hits();
	basicAll += theBasicXacts.misses();

	TmSzHistStat repAll;
	repAll += basicAll;
	repAll += theRediredReqXacts;
	repAll += theRepToRedirXacts;
	repAll += imsAll;
	repAll += theReloadXacts;

	repAll.print(os, pfx + "rep.");
	basicAll.print(os, pfx + "basic.");
	imsAll.print(os, pfx + "ims.");
	theBasicXacts.print(os, "hit", "miss", pfx);
	theRediredReqXacts.print(os, pfx + "redired_req.");
	theRepToRedirXacts.print(os, pfx + "rep_to_redir.");
	theImsXacts.print(os, "sc200", "sc304", pfx + "ims.");
	theReloadXacts.print(os, pfx + "reload.");

	theIcpXacts.print(os, "hit", "miss",  pfx + "icp.");

	theContType.print(os, pfx + "cont_type.");
	theConnClose.print(os, pfx + "conn_close.");
	theSockRdSzH.print(os, pfx + "so_read.size.");
	theSockWrSzH.print(os, pfx + "so_write.size.");

	theOidGen.print(os, pfx + "oid_gen.");
	theErrors.print(os, pfx + "errors.");

	return os;
}
