
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_TIME_AND_SIZE_HISTORGAM_STATS_H
#define PG_TIME_AND_SIZE_HISTOGRAM_STATS_H

#include "base/histograms.h"

// manages (response time, size, count) related statistics
class TmSzHistStat {
	public:
		TmSzHistStat();

		void reset();

		OLog &store(OLog &log) const;
		ILog &load(ILog &);

		const TimeHist &time() const { return theTm; }
		const SizeHist &size() const { return theSz; }

		// it is better to call this with valid tm and sz
		inline void record(const Time &tm, Size sz);

		TmSzHistStat &operator +=(const TmSzHistStat &s);

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		TimeHist theTm;
		SizeHist theSz;
};

inline OLog &operator <<(OLog &ol, const TmSzHistStat &s) { return s.store(ol); }
inline ILog &operator >>(ILog &il, TmSzHistStat &s) { return s.load(il); }


/* inlined methods */

inline
void TmSzHistStat::record(const Time &tm, Size sz) {
	if (tm >= 0)
		theTm.record(tm);
	if (sz >= 0)
		theSz.record(sz);
}

#endif
