
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_TIME_AND_SIZE_STATS_H
#define PG_TIME_AND_SIZE_STATS_H

#include "base/AggrStat.h"

class OLog;
class ILog;

// manages (response time, size, count) related statistics
class TmSzStat {
	public:
		TmSzStat();

		void reset();

		OLog &store(OLog &log) const;
		ILog &load(ILog &);
		bool sane() const { return time().sane() && size().sane(); }

		int count() const { return theTm.count(); }
		const AggrStat &time() const { return theTm; }
		const AggrStat &size() const { return theSz; }

		// it is better to call this with valid tm and sz
		inline void record(const Time &tm, Size sz);

		TmSzStat &operator +=(const TmSzStat &s);

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		AggrStat theTm;
		AggrStat theSz;
};

inline OLog &operator <<(OLog &ol, const TmSzStat &s) { return s.store(ol); }
inline ILog &operator >>(ILog &il, TmSzStat &s) { return s.load(il); }


/* inlined methods */

inline
void TmSzStat::record(const Time &tm, Size sz) {
	if (tm >= 0)
		theTm.record(tm.msec());
	if (sz >= 0)
		theSz.record(sz);
}

#endif
