
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "client/ConnHash.h"
#include "xstd/gadgets.h"


ObjFarm<ConnHashItem> ConnHash::TheItems;
ObjFarm<ConnHashQueueItem> ConnHash::TheQueueItems;


/* ConnHashItem */

ConnHashItem::ConnHashItem(): theCollision(0) {
}

void ConnHashItem::reset() {
	theCollision = 0;
}

bool ConnHashItem::vacant() const {
	return theQueue.count() == 0;
}

bool ConnHashItem::matches(const NetAddr &addr) const {
	Assert(theQueue.count());
	return theQueue.firstOut()->conn->raddr() == addr;
}


/* ConnHash */

ConnHash::ConnHash(int anCcAddrMax) {
	if (anCcAddrMax)
		ccAddrMax(anCcAddrMax);
}

void ConnHash::ccAddrMax(int aMax) {
	theHash.stretch(Max(aMax|1, 7));
	theHash.count(theHash.capacity());
	TheItems.limit(1024);      // magic, no good way to estimate
	TheQueueItems.limit(1024); // magic, no good way to estimate
}

int ConnHash::hash(const NetAddr &addr) const {
	return ((unsigned)addr.addrN().s_addr) % theHash.count();
}

bool ConnHash::find(const NetAddr &addr, ConnHashPos &pos) {
	const int idx = hash(addr);

	pos = &theHash[idx];
	while (ConnHashItem *hi = *pos) {
		if (hi->matches(addr))
			return true;
		pos = hi->nextPos();
	}
	return false;
}

bool ConnHash::find(const NetAddr &addr) {
	ConnHashPos pos;
	return find(addr, pos);
}

void ConnHash::add(Connection *conn) {
	ConnHashPos pos;
	if (!find(conn->raddr(), pos))
		*pos = TheItems.get();
	ConnHashQueueItem *qi = TheQueueItems.get();
	qi->conn = conn;
	(*pos)->queue().enqueue(qi);
}

void ConnHash::delAt(ConnHashPos pos, ConnHashQueueItem *qi) {
	TheQueueItems.put(qi);
	ConnHashItem *hi = *pos;
	if (hi->vacant()) {
		*pos = hi->collision();
		TheItems.put(hi);
	}
}

Connection *ConnHash::delAt(ConnHashPos pos) {
	Assert(*pos);
	ConnHashItem *hi = *pos;
	ConnHashQueueItem *qi = hi->queue().dequeue();
	Connection *conn = qi->conn;
	delAt(pos, qi);
	return conn;
}

void ConnHash::del(Connection *conn) {
	ConnHashPos pos;
	Assert(find(conn->raddr(), pos));
	ConnHashItem *hi = *pos;
	ConnHashQueueItem *qhead = hi->queue().firstOut();
	ConnHashQueueItem *qi = qhead;
	// XXX: this loop may be expensive if queues get long
	// perhaps merge ConnHashQueueItem with Connection somehow?
	do {
		if (qi->conn == conn) {
			hi->queue().dequeue(qi);
			delAt(pos, qi);
			return;
		}
		qi = (ConnHashQueueItem*)qi->next();
	} while (qi != qhead);
	Assert(false);
}
