
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_CONTENT_CONFIGURATION_H
#define PG_CONTENT_CONFIGURATION_H

#include "xstd/String.h"
#include "xstd/Array.h"

class ObjId;
class RndDistr;
class BodyIter;
class ContentDbase;
class TextDbase;
class ContentSym;
class EmbedContMdl;
class ObjLifeCycle;
class InjectIter;


class ContentCfg {
	public:
		typedef Array<String*> Strings;

	public:
		ContentCfg(int anId);
		~ContentCfg();

		void configure(const ContentSym *cfg);

		int id() const { return theId; }
		const String &kind() const { return theKind; }
		double newPerOid() const { return theNewPerOid; }
		const String &url_ext(int seed) const;
		const String &url_pfx(int seed) const;

		double compContPerCall(const ContentCfg *cc) const;
		void noteNewContProb(ContentCfg *cc, double newProb);
		void newPerOid(double aNewPerOid);

		BodyIter *getBodyIter(const ObjId &oid);
		void putBodyIter(BodyIter *i);
		void putInjector(InjectIter *i);

	protected:
		const String &pickStr(const Strings &strings, RndDistr *sel, int seed) const;

	public:
		String theKind;
		String theMimeType;
		RndDistr *theSize;
		ObjLifeCycle *theObjLifeCycle;
		double theChbRatio;

	protected:
		EmbedContMdl *theEmbedCont;
		ContentDbase *theCdb;

		TextDbase *theTdb;
		RndDistr *theInjGap;      // distance between injections
		double theInfProb;        // portion of infected files

		Strings theExtensions;       // file extensions
		mutable RndDistr *theExtSel; // selector for the above

		Strings thePrefixes;         // URL path prefixes
		mutable RndDistr *thePfxSel; // selector for the above

		double theNewPerOid; // new content oids per direct access oid
		int theId;
};

#endif
