
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl/pgl.h"

#include <ctype.h>

#include "xstd/gadgets.h"
#include "pgl/PglNetAddrRange.h"
#include "pgl/PglNetAddrLexer.h"


/* PglNetAddrLexer::Lexem */

PglNetAddrLexer::Lexem::Lexem(): theStart(0), theStop(0) {
}

PglNetAddrLexer::Lexem::Lexem(const char *aStart, const char *aStop):
	theStart(aStart), theStop(aStop) {
}


/* PglNetAddrLexer */

PglNetAddrLexer::PglNetAddrLexer(PglNetAddrRange *anOwner, const String &anImage):
	theLexems(anImage.len()), theOwner(anOwner), theImage(anImage) {
	Assert(theOwner && theImage);
}

bool PglNetAddrLexer::parse() {
	// split into lexems
	for (const char *p = theImage; *p;) {
		const char *start = p;
		const char ch = *start;
		if (isdigit(ch)) {
			do { ++p; } while (isdigit(*p));
			theLexems.enqueue(Lexem(start, p));
		} else
		if (ch == '.' || ch == ':' || ch == '[' || ch == ']' || ch == '-') {
			theLexems.enqueue(Lexem(start, ++p));
		} else {
			while (isalpha(*p)) p++;
			if (start == p)
				return false; // unknown char
			theLexems.enqueue(Lexem(start, p));
		}
	}

	return true;
}

char PglNetAddrLexer::charAt(int pos) const {
	if (theLexems.count() <= pos)
		return (char)0;

	return theLexems.top(pos).tag();
}

void PglNetAddrLexer::step() {
	Lexem &lm = theLexems.top();
	theOwner->addStrBlock(lm.start(), lm.stop());
	skip();
}

void PglNetAddrLexer::skip(int count) {
	while (count--)
		theLexems.dequeue();
}

int PglNetAddrLexer::intAt(int pos) const {
	Assert(pos < theLexems.count());
	int n = -1;
	Assert(isInt(theLexems.top(pos).start(), n));
	return n;
}

// int or int-int
bool PglNetAddrLexer::rangeItem() {
	if (!isdigit(charAt(0)))
		return false;
	const int start = intAt(0);

	if (charAt(+1) == '-') {

		if (!isdigit(charAt(+2)))
			return false;
		const int stop = intAt(+2) + 1;

		if (stop < start)
			return false;

		skip(3);
		theOwner->addRangeBlock(start, stop);
		return true;
	}

	skip(1);
	theOwner->addRangeBlock(start, start+1);
	return true;
}

// rangeItem, rangeItem, ...
bool PglNetAddrLexer::range() {
	bool result = false;
	while (rangeItem()) {
		result = true;
//		if (next() == ',')
//			skip();
	}
	return result;
}
