
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "xstd/xstd.h"

#include <limits.h>
#include <ctype.h>

#include "xstd/h/math.h"
#include "xstd/h/string.h"
#include "xstd/h/iostream.h"
#include "xstd/h/sys/resource.h"
#include "xstd/Time.h"

// see SSI_FD_NEWMAX below
#ifdef HAVE_SYS_SYSINFO_H
#include <sys/sysinfo.h>
#endif

#include "xstd/Assert.h"
#include "xstd/gadgets.h"


// try "ceil(700/0.7)" to see why xceil is needed
double xceil(double nom, double denom) {
#if HAVE_CEILF
	return (double)ceilf((float)(nom/denom));
#else
	const double cex = ceil(nom/denom);
	const double cm1 = cex-1;
	const double cp1 = cex+1;

	const double dm1 = Abs(nom - cm1*denom);
	const double dex = Abs(nom - cex*denom);
	const double dp1 = Abs(nom - cp1*denom);

	if (dm1 <= dex && nom <= cm1*denom )
		return dm1 <= dp1 || !(nom <= cp1*denom) ? cm1 : cp1;
	else
		return dex <= dp1 || !(nom <= cp1*denom) ? cex : cp1;
#endif
}

int XGCD(int a, int b) {
	return b ? XGCD(b, a % b) : a;
}

// strNchr(), sort of
const char *StrBoundChr(const char *s, char c, const char *eos) {
	while (s < eos) {
		if (*s == c)
			return s;
		else
			++s;
	}
	return 0;
}

const char *StrBoundAfterSpace(const char *s, const char *eos) {
	while (s < eos && !isspace(*s))
		s++;
	while (s < eos && isspace(*s))
		s++;
	return s < eos ? s : 0;
}

bool isInt(const char *s, int &i, const char **p, int base) {
	if (s) {
		char *ptr = 0;
		const int h = (int) strtol(s, &ptr, base);
		if (ptr != s && ptr) {
			i = h;
			if (p) *p = ptr;
			return true;
		}
	}
	return false;
}

bool isNum(const char *s, double &d, const char **p) {
	if (s) {
		char *ptr = 0;
		const double h = strtod(s, &ptr);
		if (ptr != s) {
			d = h;
			if (p) *p = ptr;
			return true;
		}
	}
	return false;
}

bool isDbl(const char *s, double &d, const char **p) {
	double dd;
	const char *pp;
	if (!isNum(s, dd, &pp))
		return false; 

	Assert(pp);

	// check that it is not an integer (has '.' or 'e')
	const char *h = strchr(s, '.');
	bool dbl = h && h < pp;
	if (!dbl) {
		h = strchr(s, 'e');
		dbl = h && h < pp;
	}

	if (dbl) {
		d = dd;
		if (p) *p = pp;
	}
	return dbl;
}

int xatoi(const char *s, int def) { 
	isInt(s, def);
	return def;
}

void configureStream(ostream &os, int prec) {
	os.precision(prec);
	os.setf(ios::fixed, ios::floatfield);
	os.setf(ios::showpoint, ios::basefield);
}

#ifdef HAVE_GETRLIMIT

static
int rlimit2int(rlim_t rl) {
#ifdef RLIM_INFINITY
	if (rl == RLIM_INFINITY)
		return INT_MAX;
#endif
	const int n = (int)rl;
	Should(INT_MIN <= n && n <= INT_MAX);
	return MiniMax(INT_MIN, n, INT_MAX);
}

int GetCurFD() {
	struct rlimit rl;
	if (getrlimit(RLIMIT_NOFILE, &rl) < 0)
		return -1;
	return rlimit2int(rl.rlim_cur);
}

int GetMaxFD() {
    // True64 requires this to by-pass 4K hard limit
#ifdef SSI_FD_NEWMAX
	static bool beenThere = false;
	if (!beenThere) {
		Should(setsysinfo(SSI_FD_NEWMAX, 0, 0, 0, 1) == 0);
		beenThere = true;
	}
#endif
	struct rlimit rl;
	if (getrlimit(RLIMIT_NOFILE, &rl) < 0)
		return -1;
	return rlimit2int(rl.rlim_max);
}

int SetMaxFD(int maxFD) {
	struct rlimit rl;
	rl.rlim_max = Min(maxFD, GetMaxFD());
	rl.rlim_cur = rl.rlim_max;
	if (rl.rlim_cur > 0)
		Assert(setrlimit(RLIMIT_NOFILE, &rl) >= 0);
	return rlimit2int(rl.rlim_cur);
}

#else /* HAVE_GETRLIMIT */

int GetCurFD() { return -1; }
int GetMaxFD() { return INT_MAX; }
int SetMaxFD(int maxFD) { return maxFD; }

#endif /* HAVE_GETRLIMIT */
