
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "xstd/xstd.h"

#include "xstd/h/iostream.h"

#include "xstd/Assert.h"
#include "xstd/rndDistrs.h"


UnifDistr::UnifDistr(RndGen *aGen, double aLo, double aHi):
	RndDistr(aGen), theLo(aLo), theHi(aHi) {

	Assert(theHi >= theLo);
}

ostream &UnifDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theLo << ", " << theHi << ')';
}

double ExpDistr::trial() {
	return -theMean * log(theGen->trial());
}

double NormDistr::trial() {
	double sum = 0;

	for (int i = 0; i < 12; ++i)
		sum += theGen->trial();

	return theMean + theSDev*(sum - 6.0);
}

ostream &NormDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theMean << ", " << theSDev << ')';
}

// translates mean and std dev into "internal" mu and sigma
// then creates the LognDistr object
LognDistr *LognDistr::ViaMean(RndGen *aGen, double aMean, double aSDev) {

	const double twoLnM = 2 * log(aMean);
	const double lnSum = log(aSDev*aSDev + aMean*aMean);

	const double aMu = twoLnM - lnSum/2;
	const double aSigmaSq = lnSum - twoLnM;

	return new LognDistr(aGen, aMu, aSigmaSq);
}

LognDistr::LognDistr(RndGen *generator, double aMu, double aSigmaSq):
	NormDistr(generator, aMu, sqrt(aSigmaSq)) {
}

double LognDistr::trial() {
	return ::exp(NormDistr::trial());
}

// translates "internal" mu and sigma into mean
double LognDistr::mean() const {
	return ::exp(theMean + theSDev*theSDev/2);
}

// translates "internal" mu and sigma into std dev
double LognDistr::sdev() const {
	const double w = ::exp(theSDev*theSDev);
	return sqrt(w * (w - 1) * ::exp(2*theMean));
}


/* Zipf */

// Euler-Mascheroni constant
static const double TheEMConst = 0.57721566490153286060651209;

double ZipfDistr::trial() {
//	return floor(pow(theWorldCap+1, theGen->trial()));
	const double rn = theGen->trial();
	return floor(pow(theWorldCap+1,rn));
}

int ZipfDistr::ltrial(int min, int max) {
	// wrong: (min, max]
	// return min + (int) floor(pow(max-min+1, theGen->trial()));
	// wrong: [min, max)
	// [min, max]
	min--;
	const double rn = theGen->trial();
	return min + (int) floor(pow(max-min+1, rn));
}

double ZipfDistr::omega() const {
	// harmonic sum approximation: log(i) + gamma + 1/(2i)
	return log((double)theWorldCap) + TheEMConst + 1./(2*theWorldCap);
}

ostream &ZipfDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theWorldCap << ')';
}


/* Seq */

SeqDistr::SeqDistr(RndGen *aGen, int aFirstId): RndDistr(aGen), 
	theFirstId(aFirstId), theLastId(aFirstId-1) {
}

double SeqDistr::trial() {
	return ++theLastId;
}

ostream &SeqDistr::print(ostream &os) const {
	return os << pdfName() << '(' << theFirstId << ')';
}
